@@ -1,12 +1,18 @@
 # save all the inference related functions here
 topic_init_baseline <- function(rec_data, ds_list, topic_num){
   # arrange the data stack by individual is very important as we need to make sure the matrix could be rejoined into a single matrix
-  first_incidence_age <- rec_data %>%
-    arrange(eid) %>%
-    group_by(eid, diag_icd10) %>%
-    arrange(age_diag, .by_group = T) %>% # keep only the first records of repeated diagnosis
-    dplyr::slice(1) %>%
-    dplyr::ungroup()
+  # first_incidence_age <- rec_data %>%
+  #   arrange(eid) %>%
+  #   group_by(eid, diag_icd10) %>%
+  #   arrange(age_diag, .by_group = T) %>% # keep only the first records of repeated diagnosis
+  #   dplyr::slice(1) %>%
+  #   dplyr::ungroup()
+
+  message("remove duplicated diagnoses; keep the earliest age-at-diagnosis when mulitple diagnoses are presented.")
+  data.table::setDT(rec_data)
+  # Fast + simple: sort once, keep first row per group
+  data.table::setorder(rec_data, eid, diag_icd10, age_diag)
+  first_incidence_age <- unique(rec_data, by = c("eid", "diag_icd10"))
 
   # plot the number distribution of indiviudal diseases
   df_number_records <- first_incidence_age %>%
@@ -32,23 +38,35 @@ topic_init_baseline <- function(rec_data, ds_list, topic_num){
     mutate(age_diag = round(age_diag))
 
   # the patient_list provide the column index for efficiently breaking down matrix into list of matrices
-  para$patient_lst <- para$unlist_Ds_id %>%
-    mutate(id = dplyr::row_number()) %>%
-    select(eid, id) %>%
-    group_by(eid) %>%
-    dplyr::group_split(.keep = F) %>%
-    lapply(pull)
+  # para$patient_lst <- para$unlist_Ds_id %>%
+  #   mutate(id = dplyr::row_number()) %>%
+  #   select(eid, id) %>%
+  #   group_by(eid) %>%
+  #   dplyr::group_split(.keep = F) %>%
+  #   lapply(pull)
+
+  # make it slightly faster
+  data.table::setDT(para$unlist_Ds_id)
+  data.table::set(
+    para$unlist_Ds_id,
+    j = "row_id",
+    value = seq_len(nrow(para$unlist_Ds_id))
+  )
+  para$patient_lst <- split(para$unlist_Ds_id$row_id, para$unlist_Ds_id$eid)
+
 
   para$w <- para$unlist_Ds_id %>%
     group_by(eid) %>%
     dplyr::group_split(.keep = F)
 
   # this list is splitted by disease
-  para$ds_list <- para$unlist_Ds_id %>%
-    select(-eid) %>%
-    mutate(id = dplyr::row_number()) %>%
-    group_by(Ds_id) %>%
-    dplyr::group_split()
+  # para$disease_id_list <- para$unlist_Ds_id %>%
+  #   select(-eid) %>%
+  #   mutate(id = dplyr::row_number()) %>%
+  #   group_by(Ds_id) %>%
+  #   dplyr::group_split()
+  # making this slightly faster
+  para$disease_id_list <- split(para$unlist_Ds_id$row_id, para$unlist_Ds_id$Ds_id)
 
   # initiate beta
   para$eta <- rgamma(para$D,shape = 100, rate = 100)
@@ -71,22 +89,28 @@ topic_init_baseline <- function(rec_data, ds_list, topic_num){
   # eventually reset alpha to be non-informative
   para$alpha <- rep(1, para$K)
 
-  message("ATM RAM occupation: ")
-  message(object.size(para), unit = "GB" , standard = "SI")
+  message("Rough RAM needed: ", format(4*utils::object.size(para), units = "GB", standard = "SI"))
 
   return(para)
 }
 
 topic_init_age <- function(rec_data, ds_list, topic_num, degree_free_num) {
   # arrange the data stack by individual is very important as we need to make sure the matrix could be rejoined into a single matrix
-  first_incidence_age <- rec_data %>%
-    mutate(diag_icd10 = as.character(diag_icd10)) %>% # seem to fix the bug
-    arrange(eid) %>%
-    group_by(eid, diag_icd10) %>%
-    filter(n() == 1 | age_diag == min(age_diag) ) %>% # this row is highly optimized, a lot faster the slice_min ### don't change
-    dplyr::slice(1) %>%
-    # arrange(age_diag, .by_group = T) %>% # keep only the first records of repeated diagnosis
-    dplyr::ungroup()
+  # first_incidence_age <- rec_data %>%
+  #   mutate(diag_icd10 = as.character(diag_icd10)) %>% # seem to fix the bug
+  #   arrange(eid) %>%
+  #   group_by(eid, diag_icd10) %>%
+  #   filter(n() == 1 | age_diag == min(age_diag) ) %>% # this row is highly optimized, a lot faster the slice_min ### don't change
+  #   dplyr::slice(1) %>%
+  #   # arrange(age_diag, .by_group = T) %>% # keep only the first records of repeated diagnosis
+  #   dplyr::ungroup()
+
+  # use data.table to make filtering faster
+  message("remove duplicated diagnosis; keep the earliest age-at-diagnosis when mulitple diagnosis are presented.")
+  data.table::setDT(rec_data)
+  # Fast + simple: sort once, keep first row per group
+  data.table::setorder(rec_data, eid, diag_icd10, age_diag)
+  first_incidence_age <- unique(rec_data, by = c("eid", "diag_icd10"))
 
   # plot the number distribution of indiviudal diseases
   df_number_records <- first_incidence_age %>%
@@ -113,25 +137,35 @@ topic_init_age <- function(rec_data, ds_list, topic_num, degree_free_num) {
   para$unlist_Ds_id <- first_incidence_age %>%
     mutate(Ds_id = code2id(diag_icd10)) %>%
     select(-diag_icd10) %>%
-    mutate(age_diag = round(age_diag))
+    mutate(age_diag = ceiling(age_diag))
   # the patient_list provide the column index for efficiently breaking down matrix into list of matrices
-  para$patient_lst <- para$unlist_Ds_id %>%
-    mutate(id = dplyr::row_number()) %>%
-    select(eid, id) %>%
-    group_by(eid) %>%
-    dplyr::group_split(.keep = F) %>%
-    lapply(pull)
+  # para$patient_lst <- para$unlist_Ds_id %>%
+  #   mutate(id = dplyr::row_number()) %>%
+  #   select(eid, id) %>%
+  #   group_by(eid) %>%
+  #   dplyr::group_split(.keep = F) %>%
+  #   lapply(pull)
+
+  # make it slightly faster
+  data.table::setDT(para$unlist_Ds_id)
+  data.table::set(
+    para$unlist_Ds_id,
+    j = "row_id",
+    value = seq_len(nrow(para$unlist_Ds_id))
+  )
+  para$patient_lst <- split(para$unlist_Ds_id$row_id, para$unlist_Ds_id$eid)
 
   para$w <- para$unlist_Ds_id %>%
     group_by(eid) %>%
     dplyr::group_split(.keep = F)
 
   # this list is column ID for each disease
-  para$ds_list <- para$unlist_Ds_id %>%
-    select(-eid) %>%
-    mutate(id = dplyr::row_number()) %>%
-    group_by(Ds_id) %>%
-    dplyr::group_split()
+  # para$disease_id_list <- para$unlist_Ds_id %>%
+  #   select(-eid) %>%
+  #   mutate(id = dplyr::row_number()) %>%
+  #   group_by(Ds_id) %>%
+  #   dplyr::group_split()
+  para$disease_id_list <- split(para$unlist_Ds_id$row_id, para$unlist_Ds_id$Ds_id)
 
   # create an age matrix for each disease, it is a list same length as para$w, each of Ns-by-F matrix
   para$age_max <- first_incidence_age %>%
@@ -176,8 +210,7 @@ topic_init_age <- function(rec_data, ds_list, topic_num, degree_free_num) {
   # eventually reset alpha to be non-informative
   para$alpha <- rep(1, para$K)
 
-  message("ATM RAM occupation: ")
-  message(object.size(para), unit = "GB" , standard = "SI")
+  message("Rough RAM needed: ", format(4*utils::object.size(para), units = "GB", standard = "SI"))
 
   return(para)
 }
@@ -203,8 +236,34 @@ topic_init_age <- function(rec_data, ds_list, topic_num, degree_free_num) {
 #' set.seed(1)
 #' new_weights <- loading2weights(HES_age_example[1:1000,])
 loading2weights <- function(data, ds_list = UKB_349_disease, topics = UKB_HES_10topics){
+
+  profile_size <- dim(topics)
+  message(paste0("The age profiles contain ", profile_size[3],
+                 " disease topics of ", profile_size[2], " diseases up to age ",
+                 profile_size[1]), ".")
+
+  data_not_in_disease_list <- data %>%
+    filter(!(diag_icd10 %in% ds_list$diag_icd10)) %>%
+    dim()
+  message(paste0(data_not_in_disease_list[1], " records are not in the ds_list, if this number is high it means
+                 many of the records is not covered by the comorbidity profiles."))
+
+  # if age is not in the range
+  size_outrange <- data %>%
+    filter( (age_diag <= 0) | (age_diag > profile_size[1]) ) %>%
+    dim
+  message(paste0(size_outrange[1], " records have age outside the disease range, they will be thrown away,
+                 considering using the age_imputation function."))
+  # data filtering
   data <- data %>%
-    filter(diag_icd10 %in% ds_list$diag_icd10)
+    filter(diag_icd10 %in% ds_list$diag_icd10) %>%
+    filter( age_diag > 0, age_diag <= profile_size[1])
+  if(dim(data)[1] == 0){
+    stop("No records that are covered by the profile
+         It might be the age information is missing: either use age_imputation function or
+         just assign the age_diag to a fixed number such as 50.")
+  }
+
   para <- topic_init_age(data, ds_list, dim(topics)[length(dim(topics))], degree_free_num = 5) # for internal note: degree_free_num doesn't really matter in this case
   # update beta_w: list of Ns-by-K
   para$beta_w_full <- apply(topics, 3, function(x)
@@ -348,7 +407,7 @@ fast_update_age_depend_lda <- function(para){
   para$z_zeta_sum_by_age <- sapply(min(para$unlist_Ds_id$age_diag):max(para$unlist_Ds_id$age_diag),
                                    function(y) colSums(para$z_zeta[which(para$unlist_Ds_id$age_diag == y),,drop=F]) ) %>% t
   para$phi_z <- lapply(1:para$D, function(j)
-    crossprod(para$basis_phi[para$ds_list[[j]]$id, ,drop = F], para$unlist_zn[para$ds_list[[j]]$id, ]) )  # speed up
+    crossprod(para$basis_phi[para$disease_id_list[[j]], ,drop = F], para$unlist_zn[para$disease_id_list[[j]], ]) )  # speed up
 
   #############################################################
   # # using following piece to avoid memory blowing up
@@ -408,11 +467,11 @@ age_basis_spline <- function(P, age_lst, age_min, age_max){
 lb_beta_zeta <- function(para){
   term <- sapply(1:para$K,
                  function(i)sapply(1:para$D, function(j)
-                   t(para$basis_phi[para$ds_list[[j]]$id, ] %*% para$beta[, j, i] -
-                       (rowSums(exp(para$basis_phi[para$ds_list[[j]]$id, ] %*% para$beta[, , i]) ))/
-                       para$zeta_full[para$ds_list[[j]]$id, i] -
-                       log(para$zeta_full[para$ds_list[[j]]$id, i]) + 1) %*%
-                     para$unlist_zn[para$ds_list[[j]]$id, i] ))  %>% sum
+                   t(para$basis_phi[para$disease_id_list[[j]], ] %*% para$beta[, j, i] -
+                       (rowSums(exp(para$basis_phi[para$disease_id_list[[j]], ] %*% para$beta[, , i]) ))/
+                       para$zeta_full[para$disease_id_list[[j]], i] -
+                       log(para$zeta_full[para$disease_id_list[[j]], i]) + 1) %*%
+                     para$unlist_zn[para$disease_id_list[[j]], i] ))  %>% sum
   return(term)
 }
 
@@ -456,7 +515,7 @@ lb_beta_zeta <- function(para){
 #   para$z_zeta <- para$unlist_zn/para$zeta_full # this term helps speed up
 #   para$phi_z_zeta <- lapply(1:para$K, function(i) para$basis_phi*para$z_zeta[,i]) # help to speed up
 #   para$phi_z <- lapply(1:para$D, function(j)
-#     crossprod(para$basis_phi[para$ds_list[[j]]$id, ,drop = F], para$unlist_zn[para$ds_list[[j]]$id, ]) )  # speed up
+#     crossprod(para$basis_phi[para$disease_id_list[[j]], ,drop = F], para$unlist_zn[para$disease_id_list[[j]], ]) )  # speed up
 #
 #   # para$beta <- array(sapply(1:para$K, function(i) sapply(1:para$D, function(j)
 #   #   optim(par = para$beta[,j,i],
@@ -553,7 +612,7 @@ comp_E_lntheta <- function(para){
 
 update_beta_basic_lda <- function(para){
   # compute M-step for beta: basic case, direct maximize the upper bound
-  para$beta <- sapply(1:para$D, function(j) colSums(para$unlist_zn[para$ds_list[[j]]$id,]) ) %>% t
+  para$beta <- sapply(1:para$D, function(j) colSums(para$unlist_zn[para$disease_id_list[[j]],]) ) %>% t
   # normalize beta
   para$beta <- sapply(1:para$K, function(i) para$beta[,i]/sum(para$beta[,i]))
   # this beta_w parameter save the beta for each word: it is a list of M elements and each contain a K*Ns matrix
@@ -623,6 +682,20 @@ wrapper_ATM <- function(rec_data, topic_num = 10, degree_free_num = 3, CVB_num =
   rec_data <- rec_data %>%
     filter(diag_icd10 %in% ds_list_check$diag_icd10)
 
+  # exit if data has zero age:
+  number_age_nonpositive <- rec_data %>%
+    filter(age_diag <=0) %>%
+    dim
+  if(number_age_nonpositive[1] >0){
+    stop("Age label are smaller or equal to zero; check your data frame or/and use age_imputation function.")
+  }
+  number_age_NA <- rec_data %>%
+    filter(is.na(age_diag)) %>%
+    dim
+  if(number_age_NA[1] >0){
+    stop("Age label contains NA values; check your data frame or/and use age_imputation function.")
+  }
+
   # test if all age are equal
   if(length(unique(rec_data$age_diag)) <= 1){
     message("No age variation in the data; use collapsed variational inference of LDA.")
@@ -673,97 +746,6 @@ wrapper_ATM <- function(rec_data, topic_num = 10, degree_free_num = 3, CVB_num =
 
       if(save_data){
         message(paste0("For CRAN you can't save temporary data"))
-        # message(paste0("Saving full model for CVB rep", cvb_rep))
-        # dir.create("Results")
-        # save(para, file = paste0("Results/","fullmodel_ATM_cvbrep_", cvb_rep, "K",para$K,"_P",para$P, ".RData"))
-        }
-
-    }
-    multi_runs <- list(topics, ELBOs)
-    # find the best reps in the data
-    lb_rep <- tibble::tibble(reps = as.integer(), lower_bound = as.numeric())
-    for(cvb_rep in 1:CVB_num){
-      cvrg_lb <-  ELBOs[[cvb_rep]] %>%
-        filter(!is.na(Lower_bound)) %>%
-        dplyr::slice_tail(n = 1) %>%
-        pull(2)
-      lb_rep <- lb_rep %>%
-        dplyr::add_row(reps = cvb_rep, lower_bound = cvrg_lb)
-    }
-    best_id <- order(lb_rep$lower_bound, decreasing = T)[1]
-
-    # save a smaller dataset
-    model_output <- list(topics[[best_id]], ELBOs[[best_id]], para$D, para$M, para$K, para$P, lb_rep)
-    if(save_data){
-      message(paste0("For CRAN you can't save temporary data"))
-      # save(multi_runs, file = paste0("Results/","multirun", CVB_num, "K",para$K,"_P",para$P, ".RData"))
-      # save(model_output, file = paste0("Results/","best_output_AgeLDA_RunNumber", CVB_num, "K",para$K,"_P",para$P, ".RData"))
-    }
-    output <- list()
-    output$topic_loadings <- topics[[best_id]]
-    output$topic_weights <- topic_weights[[best_id]]
-    output$ELBO_convergence <- ELBOs[[best_id]]
-    output$ds_list <- para$list_above500occu
-    output$patient_list <- para$eid
-    output$disease_number <-  para$D
-    output$patient_number <- para$M
-    output$topic_number <- para$K
-    output$topic_configuration <-para$P
-    output$multiple_run_ELBO_compare <-lb_rep
-    return(output)
-  }
-}
-
-# helper function to run LDA when there are no age information
-wrapper_LDA <- function(rec_data, topic_num, CVB_num = 5, save_data = FALSE){
-    ds_list <- rec_data %>%
-      group_by(diag_icd10) %>%
-      summarise(occ = n())
-    topics <- list()
-    ELBOs <- list()
-    topic_weights <- list()
-    for(cvb_rep in 1:CVB_num){
-      message(paste0("CVB inference number: ", cvb_rep))
-      para <- topic_init_age(rec_data, ds_list, topic_num, degree_free_num)
-      # set the number of update
-      para$max_itr <- 2000
-      para$alpha <- rep(1, para$K)
-      para$lb <- data.frame("Iteration" = 0,"Lower_bound" = CVB_lb(para))
-      para$itr_beta <- 1
-      para$itr_check_lb <- 5
-      para$itr_save <- para$max_itr + 1 # don't need to save intermediate data
-      para$tol <- 10^(-6)
-      for(itr in 1:para$max_itr){
-        message(paste0("Interation: ",itr))
-        for(itr_inside in 1:para$itr_beta){ # in practice we perform quick steps a few times before move on.
-          para <- CVB0_E_zn(para) # we choose CVB0 as papers shown it could converge quicker
-          # para <- comp_E_lntheta(para)
-        }
-
-        para <- update_beta_basic_lda(para)
-
-        # para <- update_alpha(para) # we use non-informative alpha
-        para$lb[nrow(para$lb) + 1,] <- c(itr, CVB_lb(para))
-        if(itr %% para$itr_check_lb  ==0){
-          curr_lb <- pull(filter(para$lb, Iteration == itr), Lower_bound)
-          prev_lb <- pull(filter(para$lb, Iteration == (itr -para$itr_check_lb )), Lower_bound)
-          message(paste0("Current Lower bound ", curr_lb, " at iteration: ",itr))
-          try({
-            if(is.finite((curr_lb - prev_lb)) & (curr_lb - prev_lb)/abs(prev_lb) < para$tol ){
-              message(paste0("Optimization converged at step ", itr))
-              break
-            }
-          })
-        }
-        }
-
-      topics[[cvb_rep]] <- para$beta
-      ELBOs[[cvb_rep]]  <- para$lb
-      topic_weights[[cvb_rep]] <- sweep((para$alpha_z - 1), 1, rowSums(para$alpha_z -1), FUN="/")
-
-      if(save_data){
-        message(paste0("For CRAN you can't save temporary data"))
-        # message(paste0("Saving full model for CVB rep", cvb_rep))
         # dir.create("Results")
         # save(para, file = paste0("Results/","fullmodel_ATM_cvbrep_", cvb_rep, "K",para$K,"_P",para$P, ".RData"))
       }
@@ -785,10 +767,97 @@ wrapper_LDA <- function(rec_data, topic_num, CVB_num = 5, save_data = FALSE){
     # save a smaller dataset
     model_output <- list(topics[[best_id]], ELBOs[[best_id]], para$D, para$M, para$K, para$P, lb_rep)
     if(save_data){
-      message(paste0("For CRAN you can't save temporary data"))
       # save(multi_runs, file = paste0("Results/","multirun", CVB_num, "K",para$K,"_P",para$P, ".RData"))
       # save(model_output, file = paste0("Results/","best_output_AgeLDA_RunNumber", CVB_num, "K",para$K,"_P",para$P, ".RData"))
     }
+    output <- list()
+    output$topic_loadings <- topics[[best_id]]
+    output$topic_weights <- topic_weights[[best_id]]
+    output$ELBO_convergence <- ELBOs[[best_id]]
+    output$ds_list <- para$list_above500occu
+    output$patient_list <- para$eid
+    output$disease_number <-  para$D
+    output$patient_number <- para$M
+    output$topic_number <- para$K
+    output$topic_configuration <-para$P
+    output$multiple_run_ELBO_compare <-lb_rep
+    return(output)
+  }
+}
+
+# helper function to run LDA when there are no age information
+wrapper_LDA <- function(rec_data, topic_num, CVB_num = 5, save_data = FALSE){
+  ds_list <- rec_data %>%
+    group_by(diag_icd10) %>%
+    summarise(occ = n())
+  topics <- list()
+  ELBOs <- list()
+  topic_weights <- list()
+  for(cvb_rep in 1:CVB_num){
+    message(paste0("CVB inference number: ", cvb_rep))
+    para <- topic_init_age(rec_data, ds_list, topic_num, degree_free_num)
+    # set the number of update
+    para$max_itr <- 2000
+    para$alpha <- rep(1, para$K)
+    para$lb <- data.frame("Iteration" = 0,"Lower_bound" = CVB_lb(para))
+    para$itr_beta <- 1
+    para$itr_check_lb <- 5
+    para$itr_save <- para$max_itr + 1 # don't need to save intermediate data
+    para$tol <- 10^(-6)
+    for(itr in 1:para$max_itr){
+      message(paste0("Interation: ",itr))
+      for(itr_inside in 1:para$itr_beta){ # in practice we perform quick steps a few times before move on.
+        para <- CVB0_E_zn(para) # we choose CVB0 as papers shown it could converge quicker
+        # para <- comp_E_lntheta(para)
+      }
+
+      para <- update_beta_basic_lda(para)
+
+      # para <- update_alpha(para) # we use non-informative alpha
+      para$lb[nrow(para$lb) + 1,] <- c(itr, CVB_lb(para))
+      if(itr %% para$itr_check_lb  ==0){
+        curr_lb <- pull(filter(para$lb, Iteration == itr), Lower_bound)
+        prev_lb <- pull(filter(para$lb, Iteration == (itr -para$itr_check_lb )), Lower_bound)
+        message(paste0("Current Lower bound ", curr_lb, " at iteration: ",itr))
+        try({
+          if(is.finite((curr_lb - prev_lb)) & (curr_lb - prev_lb)/abs(prev_lb) < para$tol ){
+            message(paste0("Optimization converged at step ", itr))
+            break
+          }
+        })
+      }
+    }
+
+    topics[[cvb_rep]] <- para$beta
+    ELBOs[[cvb_rep]]  <- para$lb
+    topic_weights[[cvb_rep]] <- sweep((para$alpha_z - 1), 1, rowSums(para$alpha_z -1), FUN="/")
+
+    if(save_data){
+      message(paste0("For CRAN you can't save temporary data"))
+      # dir.create("Results")
+      # save(para, file = paste0("Results/","fullmodel_ATM_cvbrep_", cvb_rep, "K",para$K,"_P",para$P, ".RData"))
+    }
+
+  }
+  multi_runs <- list(topics, ELBOs)
+  # find the best reps in the data
+  lb_rep <- tibble::tibble(reps = as.integer(), lower_bound = as.numeric())
+  for(cvb_rep in 1:CVB_num){
+    cvrg_lb <-  ELBOs[[cvb_rep]] %>%
+      filter(!is.na(Lower_bound)) %>%
+      dplyr::slice_tail(n = 1) %>%
+      pull(2)
+    lb_rep <- lb_rep %>%
+      dplyr::add_row(reps = cvb_rep, lower_bound = cvrg_lb)
+  }
+  best_id <- order(lb_rep$lower_bound, decreasing = T)[1]
+
+  # save a smaller dataset
+  model_output <- list(topics[[best_id]], ELBOs[[best_id]], para$D, para$M, para$K, para$P, lb_rep)
+  if(save_data){
+    # save(multi_runs, file = paste0("Results/","multirun", CVB_num, "K",para$K,"_P",para$P, ".RData"))
+    # save(model_output, file = paste0("Results/","best_output_AgeLDA_RunNumber", CVB_num, "K",para$K,"_P",para$P, ".RData"))
+  }
 
   output <- list()
   output$topic_loadings <- topics[[best_id]]
@@ -923,7 +992,7 @@ diseasematrix2longdata <- function(disease_matrix){
 age_imputation <- function(rec_data_missing_age, method= "mean"){
   # handling weird value
   rec_data_missing_age <- rec_data_missing_age %>%
-    mutate(age_diag = if_else((age_diag>90 | age_diag<0) , NA_real_, age_diag))
+    mutate(age_diag = if_else((age_diag>90 | age_diag<=0) , NA_real_, age_diag))
 
   if(method == "mean"){
     # first the age imputation for an individual
@@ -942,30 +1011,30 @@ age_imputation <- function(rec_data_missing_age, method= "mean"){
     age_range_per_indv <- rec_data_missing_age %>%
       group_by(eid) %>%
       dplyr::summarise(age_sub1 = suppressWarnings(max(age_diag, na.rm = T))) %>%
-      mutate(age_sub1 = if_else((age_sub1>90 | age_sub1<0) , NA_real_, age_sub1))
+      mutate(age_sub1 = if_else((age_sub1>90 | age_sub1<=0) , NA_real_, age_sub1))
     # second the age imputation for an the disease code
     age_range_per_disease <- rec_data_missing_age %>%
       group_by(diag_icd10) %>%
       summarise(age_sub2 = suppressWarnings(max(age_diag, na.rm = T))) %>%
-      mutate(age_sub2 = if_else((age_sub2>90 | age_sub2<0) , NA_real_, age_sub2))
+      mutate(age_sub2 = if_else((age_sub2>90 | age_sub2<=0) , NA_real_, age_sub2))
 
     age_pop_avg <- max(rec_data_missing_age$age_diag, na.rm = T)
-    age_pop_avg <- if_else((age_pop_avg>90 | age_pop_avg<0) , NA_real_, age_pop_avg)
+    age_pop_avg <- if_else((age_pop_avg>90 | age_pop_avg<=0) , NA_real_, age_pop_avg)
 
   }else if(method== "min"){
     # first the age imputation for an individual
     age_range_per_indv <- rec_data_missing_age %>%
       group_by(eid) %>%
       dplyr::summarise(age_sub1 = suppressWarnings(min(age_diag, na.rm = T))) %>%
-      mutate(age_sub1 = if_else((age_sub1>90 | age_sub1<0) , NA_real_, age_sub1))
+      mutate(age_sub1 = if_else((age_sub1>90 | age_sub1<=0) , NA_real_, age_sub1))
     # second the age imputation for an the disease code
     age_range_per_disease <- rec_data_missing_age %>%
       group_by(diag_icd10) %>%
       summarise(age_sub2 = suppressWarnings(min(age_diag, na.rm = T))) %>%
-      mutate(age_sub2 = if_else((age_sub2>90 | age_sub2<0) , NA_real_, age_sub2))
+      mutate(age_sub2 = if_else((age_sub2>90 | age_sub2<=0) , NA_real_, age_sub2))
 
     age_pop_avg <- min(rec_data_missing_age$age_diag, na.rm = T)
-    age_pop_avg <- if_else((age_pop_avg>90 | age_pop_avg<0) , NA_real_, age_pop_avg)
+    age_pop_avg <- if_else((age_pop_avg>90 | age_pop_avg<=0) , NA_real_, age_pop_avg)
   }
   # finding the first value from the substitute
   rec_data <- rec_data_missing_age %>%
@@ -975,5 +1044,15 @@ age_imputation <- function(rec_data_missing_age, method= "mean"){
     mutate(age_diag = dplyr::coalesce(age_diag, age_sub1, age_sub2, age_sub3)) %>%
     select(eid, diag_icd10, age_diag)
 
+  # dealing with the remaining zeros
+  number_remaining <- rec_data %>%
+    filter((age_diag>90 | age_diag<=0)) %>%
+    dim
+  message(paste0(number_remaining[1], " cases still have out of range age, they will be removed.
+                 If age is missing, we set these records to age_diag as 50 (which is the median of diagnosis age for most data sets).") )
+  rec_data <- rec_data %>%
+    mutate(age_diag = if_else(is.na(age_diag), 50, age_diag)) %>%
+    filter(age_diag<=90, age_diag>0)
+
   return(rec_data)
 }
